/*
	Copyright (C) 2011 Nicklas Nordborg

	This file is part of the Example Code Package for BASE.
	Available at http://baseplugins.thep.lu.se/
	BASE main site: http://base.thep.lu.se/
	
	This is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	The software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.examples.extensions.edit;

import java.util.Date;


import net.sf.basedb.clients.web.extensions.AbstractJspActionFactory;
import net.sf.basedb.clients.web.extensions.JspContext;
import net.sf.basedb.clients.web.extensions.edit.AbstractOnSaveAction;
import net.sf.basedb.clients.web.extensions.edit.DebugOnSaveActionFactory.DebugOnSaveAction;
import net.sf.basedb.clients.web.extensions.edit.OnSaveAction;
import net.sf.basedb.core.AnyToAny;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.User;
import net.sf.basedb.util.extensions.InvokationContext;

/**
	Example on-save action factory that creates an any-to-any link
	between any item and the currently logged in user. Eg.
	a simple logging feature that stores information about the 
	last update (using the web interface).
	
	@author nicklas
*/
public class LastUpdatedByOnSaveActionFactory 
	extends AbstractJspActionFactory<OnSaveAction>
{

	public LastUpdatedByOnSaveActionFactory()
	{}

	/*
		From the ActionFactory interface
		--------------------------------
	*/
	@Override
	public OnSaveAction[] getActions(InvokationContext<? super OnSaveAction> context)
	{
		JspContext jsp = (JspContext)context.getClientContext();
		OnSaveAction action = null;
		if (jsp.getGuiContext().getItem().getItemClass() == null)
		{
			// Items that doesn't have an item-class (eg. reporter) 
			// can't be used with AnyToAny -- print some debug info instead
			action = new DebugOnSaveAction(jsp);
		}
		else
		{
			action = new LastUpdatedByOnSaveAction(jsp);
		}
		return new OnSaveAction[] { action };
	}
	// ---------------------------
	
	
	/**
		The action class is doing the actual work. We extend
		AbstractOnSaveAction for future compatibility.
	*/
	public static class LastUpdatedByOnSaveAction
		extends AbstractOnSaveAction
	{
	
		/**
			Create a new action for the given context.
			The current item can be retreived by {@link JspContext#getCurrentItem()}.
		*/
		public LastUpdatedByOnSaveAction(JspContext context)
		{
			super(context);
		}

		/*
			From the OnSaveAction interface
			-------------------------------
		*/
		@Override
		public void onSave()
		{
			JspContext jsp = getJspContext();
			BasicItem item = (BasicItem)jsp.getCurrentItem();
			DbControl dc = jsp.getDbControl();
			User user = User.getById(dc, dc.getSessionControl().getLoggedInUserId());
			AnyToAny any = AnyToAny.getNewOrExisting(dc, item, "last-updated-by", user, false);
			any.setDescription("On " + new Date());
			if (!any.isInDatabase()) dc.saveItem(any);
		}
		// -------------------------------------
	}
	
}
